import sys


class ZillowError(Exception):
    """
    Error messages copied from Zillow\'s API documentation
    http://www.zillow.com/howto/api/GetDeepSearchResults.htm
    """
    code = dict([
        (0, 'Request successfully processed'),
        (1, 'Service error-there was a server-side error ' +
            'while processing the request. \n ' +
            'Check to see if your url is properly formed: delimiters, ' +
            'character cases, etc.'),
        (2, 'The specified ZWSID parameter was invalid or ' +
            'not specified in the request. \n' +
            'Check if you have provided a ZWSID in your API call. ' +
            'If yes, check if the ZWSID is keyed in correctly. ' +
            'If it still doesn\'t work, contact Zillow to ' +
            'get help on fixing your ZWSID.'),
        (3, 'Web services are currently unavailable.\n' +
            'The Zillow Web Service is currently not available. ' +
            'Please come back later and try again.'),
        (4, 'The API call is currently unavailable.\n' +
            'The Zillow Web Service is currently not available. ' +
            'Please come back later and try again.'),
        (500, 'Invalid or missing address parameter.\n'
            'Check if the input address parameter ' +
            'matches the format specified ' +
            'in the input parameters table. When inputting a city name, the ' +
            'state should also be given. A city name alone will not give ' +
            'a valid address.'),
        (501, 'Invalid or missing citystatezip parameter. ' +
            'Check if the input address parameter matches ' +
            'the format specified ' +
            'in the input parameters table. When inputting a city name, the ' +
            'state should also be given. A city name alone will not give' +
            'a valid address.'),
        (502, 'No results found. \n ' +
            'Sorry, the address you provided is not found in the Zillow ' +
            'property database.'),
        (503, 'Failed to resolve city, state or ZIP code.\n ' +
            'Check if the city-state combination is valid. ' +
            'Also check if you have provided a valid ZIP code.'),
        (504, 'No coverage for specified area. \nThe specified area ' +
            'is not covered by the Zillow property database.'),
        (505, 'Timeout Your request timed out. \nThe server could be ' +
            'busy or unavailable. Try again later.'),
        (506, 'Address string too long. \nIf address is valid, try using ' +
            'abbreviations.'),
        (507, 'No exact match found. \n' +
            'Verify that the given address is correct.'),
        # this error isn't specified in the zillow doc's,
        # but returned instead of 502
        (508, 'No exact match found for input address.'),
    ])

    def __init__(self, status, url=None, response=None):
        """

        """
        Exception.__init__(self, status)  # Exception is an old-school class
        self.status = status
        self.message = {
            'code': status,
            'text': self.code[int(status)]}
        self.url = url
        self.response = response

    def __unicode__(self):
        return self.message

    if sys.version_info[0] >= 3:  # Python 3
        def __str__(self):
            return self.__unicode__()
    else:  # Python 2
        def __str__(self):
            return self.__unicode__().encode('utf8')


class ZillowFail(Exception):

    def __init__(self):
        Exception.__init__(self)


class ZillowNoResults(Exception):

    def __init__(self):
        Exception.__init__(self)



#!/usr/bin/env python
#

"""
Tests for `pyzillow` module.
"""

import responses
from pytest import raises
from pyzillow.pyzillow import (
    ZillowWrapper,
    GetDeepSearchResults,
    GetUpdatedPropertyDetails
)
from pyzillow.pyzillowerrors import ZillowError


class TestPyzillow(object):

    @classmethod
    def setup_class(cls):
        cls.ZILLOW_API_KEY = 'X1-ZWz1b88d9eaq6j_1wtus'
        cls.address = '2114 Bigelow Ave Seattle, WA'
        cls.zipcode = '98109'

    def test_zillow_api_connect(self):
        # create response from zillow and check for error code '0'
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        zillow_search_response = zillow_data.get_deep_search_results(
            self.address, self.zipcode)
        assert \
            zillow_search_response.find('message').find('code').text == '0'

    def test_zillow_error_invalid_ZWSID(self):
        # This test checks the correct error message if no ZWSID is provided.
        # Expected error code: 2
        zillow_data = ZillowWrapper(None)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=self.address,
            zipcode=self.zipcode)

    def test_zillow_error_missing_address(self):
        # This test checks the correct error message if no address is provided.
        # Expected error code: 500
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=None,
            zipcode=self.zipcode)

    def test_zillow_error_missing_zipcode(self):
        # This test checks the correct error message if no zipcode is provided.
        # Expected error code: 501
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=self.address,
            zipcode=None)

    def test_zillow_error_no_property_match(self):
        # This test checks the correct error message if no property is found.
        # Expected error code: 508
        # Address and zip code of an exisiting property, but not listed
        address = '599 Pennsylvania Avenue Northwest, Washington, DC'
        zipcode = '20001'
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=address,
            zipcode=zipcode)

    def test_zillow_error_match_zipcode_city(self):
        # This test checks the correct error message if no zipcode is provided.
        # Expected error code: 503
        mismatch_zipcode = '97204'  # Portland, OR
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=self.address,
            zipcode=mismatch_zipcode)

    def test_zillow_error_invalid_zipcode(self):
        # This test checks the correct error message if an
        # invalid zipcode is provided.
        # Expected error code: 503
        invalid_zipcode = 'ABCDE'  # invalid zipcode
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=self.address,
            zipcode=invalid_zipcode)

    def test_zillow_error_no_coverage(self):
        # This test checks the correct error message
        # if no coverage is provided.
        # Expected error code: 504
        address = 'Calle 21 y O, Vedado, Plaza, Ciudad de la Habana, Cuba'
        zipcode = '10400'  # Cuban address, I assume Zillow doesn't cover Cuba
        zillow_data = ZillowWrapper(self.ZILLOW_API_KEY)
        raises(
            ZillowError,
            zillow_data.get_deep_search_results,
            address=address,
            zipcode=zipcode)

    @responses.activate
    def test_deep_search_results(self):
        """
        """

        address = '2114 Bigelow Ave Seattle, WA'
        zipcode = '98109'

        responses.add(
            responses.GET,
            'http://www.zillow.com/webservice/GetDeepSearchResults.htm',
            body='<?xml version="1.0" encoding="utf-8"?><SearchResults:searchresults xsi:schemaLocation="http://www.zillow.com/static/xsd/SearchResults.xsd http://www.zillowstatic.com/vstatic/34794f0/static/xsd/SearchResults.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SearchResults="http://www.zillow.com/static/xsd/SearchResults.xsd"><request><address>2114 Bigelow Ave Seattle, WA</address><citystatezip>98109</citystatezip></request><message><text>Request successfully processed</text><code>0</code></message><response><results><result><zpid>48749425</zpid><links><homedetails>http://www.zillow.com/homedetails/2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/</homedetails><graphsanddata>http://www.zillow.com/homedetails/2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/#charts-and-data</graphsanddata><mapthishome>http://www.zillow.com/homes/48749425_zpid/</mapthishome><comparables>http://www.zillow.com/homes/comps/48749425_zpid/</comparables></links><address><street>2114 Bigelow Ave N</street><zipcode>98109</zipcode><city>Seattle</city><state>WA</state><latitude>47.637933</latitude><longitude>-122.347938</longitude></address><FIPScounty>53033</FIPScounty><useCode>SingleFamily</useCode><taxAssessmentYear>2014</taxAssessmentYear><taxAssessment>1060000.0</taxAssessment><yearBuilt>1924</yearBuilt><lotSizeSqFt>4680</lotSizeSqFt><finishedSqFt>3470</finishedSqFt><bathrooms>3.0</bathrooms><bedrooms>4</bedrooms><lastSoldDate>11/26/2008</lastSoldDate><lastSoldPrice currency="USD">1025000</lastSoldPrice><zestimate><amount currency="USD">1419804</amount><last-updated>09/10/2015</last-updated><oneWeekChange deprecated="true"></oneWeekChange><valueChange duration="30" currency="USD">20690</valueChange><valuationRange><low currency="USD">1292022</low><high currency="USD">1547586</high></valuationRange><percentile>0</percentile></zestimate><localRealEstate><region name="East Queen Anne" id="271856" type="neighborhood"><zindexValue>629,900</zindexValue><links><overview>http://www.zillow.com/local-info/WA-Seattle/East-Queen-Anne/r_271856/</overview><forSaleByOwner>http://www.zillow.com/east-queen-anne-seattle-wa/fsbo/</forSaleByOwner><forSale>http://www.zillow.com/east-queen-anne-seattle-wa/</forSale></links></region></localRealEstate></result></results></response></SearchResults:searchresults><!-- H:003  T:27ms  S:1181  R:Sat Sep 12 23:30:47 PDT 2015  B:4.0.19615-release_20150908-endor.2fa5797~candidate.358c83d -->',
            content_type='application/xml',
            status=200
        )

        zillow_data = ZillowWrapper(api_key=None)
        deep_search_response = zillow_data.get_deep_search_results(
            address, zipcode)
        result = GetDeepSearchResults(deep_search_response)

        assert result.zillow_id == '48749425'
        assert result.home_type == 'SingleFamily'
        assert result.home_detail_link == \
            'http://www.zillow.com/homedetails/' + \
            '2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/'
        assert result.graph_data_link == \
            'http://www.zillow.com/homedetails/' + \
            '2114-Bigelow-Ave-N-Seattle-WA-98109/' + \
            '48749425_zpid/#charts-and-data'
        assert result.map_this_home_link == \
            'http://www.zillow.com/homes/48749425_zpid/'
        assert result.tax_year == '2014'
        assert result.tax_value == '1060000.0'
        assert result.year_built == '1924'
        assert result.property_size == '4680'
        assert result.home_size == '3470'
        assert result.bathrooms == '3.0'
        assert result.bedrooms == '4'
        assert result.last_sold_date == '11/26/2008'
        assert result.last_sold_price_currency == 'USD'
        assert result.last_sold_price == '1025000'
        lat = float(result.latitude)
        assert lat - 0.01 <= 47.637933 <= lat + 0.01
        lng = float(result.longitude)
        assert lng - 0.01 <= -122.347938 <= lng + 0.01
        assert result.zestimate_amount == '1419804'
        # assert result.zestimate_currency == 'USD'
        assert result.zestimate_last_updated == '09/10/2015'
        assert result.zestimate_value_change == '20690'
        assert result.zestimate_valuation_range_high == '1547586'
        assert result.zestimate_valuationRange_low == '1292022'
        assert result.zestimate_percentile == '0'

    @responses.activate
    def test_get_updated_property_details_results(self):
        """
        """

        zillow_id = '48749425'
        responses.add(
            responses.GET,
            'http://www.zillow.com/webservice/GetUpdatedPropertyDetails.htm',
            body='<?xml version="1.0" encoding="utf-8"?><UpdatedPropertyDetails:updatedPropertyDetails xmlns:UpdatedPropertyDetails="http://www.zillow.com/static/xsd/UpdatedPropertyDetails.xsd" xsi:schemaLocation="http://www.zillow.com/static/xsd/UpdatedPropertyDetails.xsd http://www.zillowstatic.com/vstatic/34794f0/static/xsd/UpdatedPropertyDetails.xsd" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"><request><zpid>48749425</zpid></request><message><text>Request successfully processed</text><code>0</code></message><response><zpid>48749425</zpid><pageViewCount><currentMonth>16125</currentMonth><total>16125</total></pageViewCount><address><street>2114 Bigelow Ave N</street><zipcode>98109</zipcode><city>Seattle</city><state>WA</state><latitude>47.637933</latitude><longitude>-122.347938</longitude></address><links><homeDetails>http://www.zillow.com/homedetails/2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/</homeDetails><photoGallery>http://www.zillow.com/homedetails/2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/#image=lightbox%3Dtrue</photoGallery><homeInfo>http://www.zillow.com/homedetails/2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/</homeInfo></links><images><count>1</count><image><url>http://photos3.zillowstatic.com/p_d/ISxb3qa8s1cwx01000000000.jpg</url></image></images><editedFacts><useCode>SingleFamily</useCode><bedrooms>4</bedrooms><bathrooms>3.0</bathrooms><finishedSqFt>3470</finishedSqFt><lotSizeSqFt>4680</lotSizeSqFt><yearBuilt>1924</yearBuilt><yearUpdated>2003</yearUpdated><numFloors>2</numFloors><basement>Finished</basement><roof>Composition</roof><view>Water, City, Mountain</view><parkingType>Off-street</parkingType><heatingSources>Gas</heatingSources><heatingSystem>Forced air</heatingSystem><rooms>Laundry room, Walk-in closet, Master bath, Office, Dining room, Family room, Breakfast nook</rooms></editedFacts><neighborhood>Queen Anne</neighborhood><schoolDistrict>Seattle</schoolDistrict><elementarySchool>John Hay</elementarySchool><middleSchool>McClure</middleSchool></response></UpdatedPropertyDetails:updatedPropertyDetails><!-- H:001  T:127ms  S:990  R:Sat Sep 12 23:47:31 PDT 2015  B:4.0.19615-release_20150908-endor.2fa5797~candidate.358c83d -->',
            content_type='application/xml',
            status=200
        )

        zillow_data = ZillowWrapper(api_key=None)
        updated_property_details_response = \
            zillow_data.get_updated_property_details(zillow_id)
        result = GetUpdatedPropertyDetails(updated_property_details_response)

        assert result.zillow_id == '48749425'
        assert result.home_type == 'SingleFamily'
        assert result.home_detail_link == \
            'http://www.zillow.com/homedetails/' + \
            '2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/'
        assert result.photo_gallery == \
            'http://www.zillow.com/homedetails/' + \
            '2114-Bigelow-Ave-N-Seattle-WA-98109/' + \
            '48749425_zpid/#image=lightbox%3Dtrue'
        assert result.home_info == \
            'http://www.zillow.com/homedetails/' + \
            '2114-Bigelow-Ave-N-Seattle-WA-98109/48749425_zpid/'
        assert result.year_built == '1924'
        assert result.property_size == '4680'
        assert result.home_size == '3470'
        assert result.bathrooms == '3.0'
        assert result.bedrooms == '4'
        assert result.year_updated == '2003'
        assert result.basement == 'Finished'
        assert result.roof == 'Composition'
        assert result.view == 'Water, City, Mountain'
        assert result.heating_sources == 'Gas'
        assert result.heating_system == 'Forced air'
        assert result.rooms == \
            'Laundry room, Walk-in closet, Master bath, Office,' + \
            ' Dining room, Family room, Breakfast nook'
        assert result.neighborhood == 'Queen Anne'
        assert result.school_district == 'Seattle'
        lat = float(result.latitude)
        assert lat - 0.01 <= 47.637933 <= lat + 0.01
        lng = float(result.longitude)
        assert lng - 0.01 <= -122.347938 <= lng + 0.01
        assert result.floor_material is None
        assert result.num_floors == '2'
        assert result.parking_type == 'Off-street'
        # assert result.home_description == """Bright, spacious, """

    @classmethod
    def teardown_class(cls):
        pass



import sys
import requests

from xml.etree import cElementTree as ElementTree  # for zillow API

from .pyzillowerrors import ZillowError, ZillowFail, ZillowNoResults
import pyzillow


class ZillowWrapper(object):
    """
    """

    def __init__(self, api_key=None):
        """

        """
        self.api_key = api_key

    def get_deep_search_results(self, address, zipcode):
        """
        GetDeepSearchResults API
        """

        url = 'http://www.zillow.com/webservice/GetDeepSearchResults.htm'
        params = {
            'address': address,
            'citystatezip': zipcode,
            'zws-id': self.api_key
        }
        return self.get_data(url, params)

    def get_updated_property_details(self, zpid):
        """
        GetUpdatedPropertyDetails API
        """
        url = 'http://www.zillow.com/webservice/GetUpdatedPropertyDetails.htm'

        params = {
            'zpid': zpid,
            'zws-id': self.api_key
        }
        return self.get_data(url, params)

    def get_data(self, url, params):
        """
        """

        try:
            request = requests.get(
                url=url,
                params=params,
                headers={
                    'User-Agent': ''.join([
                        'pyzillow/',
                        pyzillow.__version__,
                        ' (Python)'
                    ])
                })
        except (
            requests.exceptions.ConnectionError,
            requests.exceptions.TooManyRedirects,
                requests.exceptions.Timeout):
            raise ZillowFail

        try:
            request.raise_for_status()
        except requests.exceptions.HTTPError:
            raise ZillowFail

        try:
            response = ElementTree.fromstring(request.text)
        except ElementTree.ParseError:
            print (
                "Zillow response is not a valid XML (%s)" % (
                    params['address']
                )
            )
            raise ZillowFail

        if response.findall('message/code')[0].text is not '0':
            raise ZillowError(int(response.findall('message/code')[0].text))
        else:
            if not response.findall('response'):
                print (
                    "Zillow returned no results for (%s)" % (
                        params['address']
                    )
                )
                raise ZillowNoResults
            return response


class ZillowResults(object):
    """
    """

    attribute_mapping = {}

    def get_attr(self, attr):
        """
        """
        try:
            return self.data.find(self.attribute_mapping[attr]).text
        except AttributeError:
            return None

    def __unicode__(self):
        return self.zillow_id

    if sys.version_info[0] >= 3:  # Python 3
        def __str__(self):
            return self.__unicode__()

    else:  # Python 2
        def __str__(self):
            return self.__unicode__().encode('utf8')

    @property
    def area_unit(self):
        """
        lotSizeSqFt
        """
        return u'SqFt'

    @property
    def last_sold_price_currency(self):
        """
        lastSoldPrice currency
        """
        return self.data.find(
            self.attribute_mapping['last_sold_price']).attrib["currency"]


class GetDeepSearchResults(ZillowResults):
    """
    """
    attribute_mapping = {
        'zillow_id': 'result/zpid',
        'home_type': 'result/useCode',
        'home_detail_link': 'result/links/homedetails',
        'graph_data_link': 'result/links/graphsanddata',
        'map_this_home_link': 'result/links/mapthishome',
        'latitude': 'result/address/latitude',
        'longitude': 'result/address/longitude',
        'tax_year': 'result/taxAssessmentYear',
        'tax_value': 'result/taxAssessment',
        'year_built': 'result/yearBuilt',
        'property_size': 'result/lotSizeSqFt',
        'home_size': 'result/finishedSqFt',
        'bathrooms': 'result/bathrooms',
        'bedrooms': 'result/bedrooms',
        'last_sold_date': 'result/lastSoldDate',
        'last_sold_price': 'result/lastSoldPrice',
        'zestimate_amount': 'result/zestimate/amount',
        'zestimate_last_updated': 'result/zestimate/last-updated',
        'zestimate_value_change': 'result/zestimate/valueChange',
        'zestimate_valuation_range_high':
        'result/zestimate/valuationRange/high',
        'zestimate_valuationRange_low': 'result/zestimate/valuationRange/low',
        'zestimate_percentile': 'result/zestimate/percentile',
    }

    def __init__(self, data, *args, **kwargs):
        """
        Creates instance of GeocoderResult from the provided XML data array
        """
        self.data = data.findall('response/results')[0]
        for attr in self.attribute_mapping.__iter__():
            try:
                self.__setattr__(attr, self.get_attr(attr))
            except AttributeError:
                print ('AttributeError with %s' % attr)


class GetUpdatedPropertyDetails(ZillowResults):
    """
    """
    attribute_mapping = {
        # attributes in common with GetDeepSearchResults
        'zillow_id': 'zpid',
        'home_type': 'editedFacts/useCode',
        'home_detail_link': 'links/homeDetails',
        'graph_data_link': '',
        'map_this_home_link': '',
        'latitude': 'address/latitude',
        'longitude': 'address/longitude',
        'tax_year': '',
        'tax_value': '',
        'year_built': 'editedFacts/yearBuilt',
        'property_size': 'editedFacts/lotSizeSqFt',
        'home_size': 'editedFacts/finishedSqFt',
        'bathrooms': 'editedFacts/bathrooms',
        'bedrooms': 'editedFacts/bedrooms',
        'last_sold_date': '',
        'last_sold_price': '',
        # new attributes in GetUpdatedPropertyDetails
        'photo_gallery': 'links/photoGallery',
        'home_info': 'links/homeInfo',
        'year_updated': 'editedFacts/yearUpdated',
        'floor_material': 'editedFacts/floorCovering',
        'num_floors': 'editedFacts/numFloors',
        'basement': 'editedFacts/basement',
        'roof': 'editedFacts/roof',
        'view': 'editedFacts/view',
        'parking_type': 'editedFacts/parkingType',
        'heating_sources': 'editedFacts/heatingSources',
        'heating_system': 'editedFacts/heatingSystem',
        'rooms': 'editedFacts/rooms',
        'num_rooms': 'editedFacts/numRooms',
        'appliances': 'editedFacts/appliances',
        'neighborhood': 'neighborhood',
        'school_district': 'schoolDistrict',
        'home_description': 'homeDescription',
    }

    def __init__(self, data, *args, **kwargs):
        """
        Creates instance of GeocoderResult from the provided XML data array
        """
        self.data = data.findall('response')[0]
        for attr in self.attribute_mapping.__iter__():
            try:
                self.__setattr__(attr, self.get_attr(attr))
            except AttributeError:
                print ('AttributeError with %s' % attr)
